import copy
import numpy as np
import networkx as nx
import random
import matplotlib.pyplot as plt


def remove_items(test_list, item_to_remove):
    for index in test_list:
        if (index == item_to_remove):
            test_list.remove(index)
    return test_list


max_reps = 250

for method in ['Degenerate', 'NoDegenerate']:
    if method == 'Degenerate':
        G0 = nx.read_pajek('LichenGuilds_degeneracy.pajek')
    else:
        G0 = nx.read_pajek('LichenGuilds_nodegeneracy.pajek')

        print(method, size)
        types = nx.get_node_attributes(G0, "type")

        C1 = []
        C2 = []
        list_photobionts = []
        list_mycobionts = []
        for nodea in G0.nodes():
            if types[nodea] == "Photobiont":
                if nodea not in list_photobionts:
                    list_photobionts.append(nodea)
                for nodeb in G0.nodes():
                    if types[nodeb] == "Mycobiont":
                        if nodeb not in list_mycobionts:
                            list_mycobionts.append(nodeb)
                        if G0.has_edge(nodea, nodeb):
                            C1.append(nodea)
                            C2.append(nodeb)

        num_edges = len(C1)
        num_ph = len(list_photobionts)
        num_mb = len(list_mycobionts)
        num_sp = num_ph + num_mb
        print('species and edges:', num_ph, num_mb, num_sp, num_edges)

        # Edgelist bootstrap
        rep = 0
        while rep <= max_reps:
            random.shuffle(C1)
            random.shuffle(C2)
            forbidden = {}
            edgelist = []
            C2working = copy.deepcopy(C2)
            for i in range(len(C1)):
                C2copy = copy.deepcopy(C2working)
                if C1[i] in forbidden:
                    for item in forbidden[C1[i]]:
                        C2copy = remove_items(C2copy, item)
                else:
                    forbidden[C1[i]] = []
                try:
                    choice = random.choice(range(len(C2copy)))
                    if choice != "":
                        forbidden[C1[i]].append(C2copy[choice])
                        edgelist.append((C1[i], C2copy[choice]))
                        C2working.remove(C2copy[choice])
                except:
                    continue

            G_rep = nx.Graph(edgelist)
            num_edges_rep = len(G_rep.edges())
            if num_edges_rep == num_edges:
                rep += 1
                nx.write_pajek(G_rep, "./NullModels/NullModels_" + method + "_" + size + "/BS_" + str(rep) + ".pajek")
            else:
                print('Failure', len(edgelist), num_edges, num_edges_rep)

