#Code by Salva Duran-Nebreda

import copy
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
import networkx as nx
import random

Graph = nx.Graph(nx.read_pajek('LichenGuilds_Degenerate_Full.pajek'))
Gcc = sorted(nx.connected_components(Graph), key=len, reverse=True)
G0 = Graph.subgraph(Gcc[0])
N = G0.number_of_nodes()
list_of_nodes = [x for x in G0.nodes]

dims = ['Method', 'Repeat', 'log(probability of propagation)', 'Surviving fraction']
df = pd.DataFrame(columns=dims)

#BetweennessCentrality = nx.algorithms.centrality.betweenness_centrality(Graph, normalized=True)

for repeat in range(100):
    p = 1
    Graph_BS = nx.Graph(nx.read_pajek('./NullModels_Degenerate_Full/BS_' + str(repeat + 1) + '.pajek'))
    Gcc = sorted(nx.connected_components(Graph_BS), key=len, reverse=True)
    G0_BS = Graph_BS.subgraph(Gcc[0])
    list_of_nodes_bs = [x for x in G0_BS.nodes]
    N_BS = G0_BS.number_of_nodes()

    print(repeat, N, N_BS)

    while p >= 0.05:
        stack = [random.choice(list_of_nodes)]
        infected = []
        while len(stack) > 0:
            subject = stack.pop(0)
            neighbors = G0.neighbors(subject)
            for neighbor in neighbors:
                if neighbor not in infected and neighbor not in stack and random.random() < p:
                    stack.append(neighbor)
            infected.append(subject)

        df_ = pd.DataFrame([['Data', repeat, np.log(p), 1 - len(infected) / N]], columns=dims)
        df = pd.concat((df, df_))

        stack = [random.choice(list_of_nodes_bs)]
        infected = []
        while len(stack) > 0:
            subject = stack.pop(0)
            neighbors = G0_BS.neighbors(subject)
            for neighbor in neighbors:
                if neighbor not in infected and neighbor not in stack and random.random() < p:
                    stack.append(neighbor)
            infected.append(subject)

        df_ = pd.DataFrame([['Bootstrap',repeat,np.log(p),1-len(infected)/N_BS]], columns=dims)
        df = pd.concat((df,df_))

        p = round(p / 1.01,4)

df.to_csv('Data_cascade_resilience.csv')
