#Code by Salva DUran-Nebreda

import copy
import random
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
import networkx as nx


def unique(numbers):
    lun = []
    for number in numbers:
        if number not in lun:
            lun.append(number)
    return lun


#Murata bipartite modularity
def Murata_biM(Bi_Graph, partition, node_type):
    Qb = 0
    num_edges = Bi_Graph.number_of_edges()
    for community in unique(partition):
        CN = []
        i = 0
        for node in Bi_Graph.nodes():
            if partition[i] == community:
                CN.append(node)
            i += 1
        CSG = Bi_Graph.subgraph(CN)
        I = CSG.number_of_edges()
        ka = 0
        kb = 0
        for node in CSG.nodes():
            #print(node)
            if node_type[node] == "A":
                ka += Bi_Graph.degree(node)
            else:
                kb += Bi_Graph.degree(node)
        Qb += I/num_edges - ka * kb/(num_edges**2)
    return Qb


dims = ['Repeat', 'Method', 'QB', 'NumModules']
df = pd.DataFrame(columns=dims)
G0 = nx.read_pajek('LichenGuilds_Degenerate_Full.pajek')


#Loop over null models
for rep in range(100):
    for method in ['BS', 'ER', 'Data']:
        if method != 'Data':
            # Read pajek
            filename = "./NullModels_Degenerate_Full/"+method+"_" + str(rep+1) + ".pajek"
            Graph = nx.read_pajek(filename)
        else:
            Graph = nx.Graph(G0)

        #Establish bipartiteness
        node_type = nx.get_node_attributes(G0, 'type')
        for node in node_type:
            #print(node)
            if node_type[node] == 'Mycobiont':
                node_type[node] = 'A'
            if node_type[node] == 'Photobiont':
                node_type[node] = 'B'

        #Create the initial random partition
        num_C = Graph.number_of_nodes()
        partition = list()
        community_assignation = dict()

        i = 0
        for j in Graph.nodes:
            partition.append(i)
            community_assignation[j] = i
            i += 1

        Q_ = -1
        testQ_ = Murata_biM(Graph, partition, node_type)
        deltaQ = 1
        ask = [x for x in Graph if len(list(Graph.neighbors(x)))>0]

        while deltaQ > 0.000001:
            random.shuffle(ask)

            for node in ask:
                oldpartition = copy.deepcopy(partition)
                currentC = copy.deepcopy(community_assignation[node])
                '''target = list(range(num_C))
                target.remove(currentC)'''
                target = [community_assignation[neighbor] for neighbor in Graph.neighbors(node)]
                if len(target) > 0:
                    best_move = dict()
                    for swapC in target:
                        #community_assignation[node] = copy.deepcopy(swapC)
                        partition[list(Graph.nodes).index(node)] = swapC

                        testQ = Murata_biM(Graph, partition, node_type)
                        best_move[swapC] = testQ

                    #community_assignation[node] = copy.deepcopy(currentC)
                    partition = copy.deepcopy(oldpartition)

                    best = max([best_move[key] for key in best_move])
                    for key in best_move:
                        if best_move[key] == best and best > testQ_:
                            community_assignation[node] = key
                            partition[list(Graph.nodes).index(node)] = key

                            testQ = Murata_biM(Graph, partition, node_type)
                            #print("\t", len(set(partition)), testQ, testQ_, testQ - testQ_)
                            testQ_ = copy.deepcopy(testQ)

            Q = Murata_biM(Graph, partition, node_type)
            deltaQ = Q - Q_
            print(method, rep, deltaQ)
            Q_ = copy.deepcopy(Q)
        df_ = pd.DataFrame([[rep, method, Q, len(set(partition))]], columns=dims)
        df = pd.concat((df, df_))

    '''positions = nx.spring_layout(Graph)
    nx.draw(Graph, positions, cmap=plt.get_cmap('viridis'), node_color=partition)
    nx.draw_networkx_labels(Graph, positions, community_assignation)
    plt.show()
    plt.close()'''
df.to_csv('biLOUVAIN_Modularity_Degeneracy_Full_LAST.csv')