#Code by Salva Duran-Nebreda

import copy
import random
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
import networkx as nx
import nodf

dims = ['Analysis','Repeat', 'Method', 'WholeNest', 'ColNest', 'RowNest']

#Loop over null models
for analysis_type in reversed(['Degenerate_Full']):#, 'Degenerate_GCC', 'NoDegenerate_Full', 'NoDegenerate_GCC']):
    df = pd.DataFrame(columns=dims)
    G0 = nx.read_pajek('LichenGuilds_'+analysis_type+'.pajek')
    node_type = nx.get_node_attributes(G0, 'type')
    for rep in range(100):
        print(analysis_type, rep)
        for method in ['BS', 'ER', 'Data']:
            if method != 'Data':
                # Read pajek
                filename = "./NullModels_"+analysis_type+"/"+method+"_" + str(rep+1) + ".pajek"
                Graph = nx.read_pajek(filename)
            else:
                Graph = nx.Graph(G0)

            #node_type = nx.get_node_attributes(Graph,'type')
            nP = [node for node in node_type if node_type[node] == 'Photobiont']
            nM = [node for node in node_type if node_type[node] == 'Mycobiont']
            M = np.zeros((len(nP),len(nM)))
            for i, node1 in enumerate(nP):
                for j, node2 in enumerate(nM):
                    if Graph.has_edge(node1,node2):
                        M[i,j] = 1

            #Compute Nestedness
            [WholeNest, ColNest, RowNest] = nodf.nodf(M)

            df_ = pd.DataFrame([[analysis_type, rep, method, WholeNest, ColNest, RowNest]], columns=dims)
            df = pd.concat((df, df_))


    df.to_csv('Nestedness_'+analysis_type+'_LAST.csv')